@@ -17,9 +17,9 @@ int main( int /*argc*/, char ** argv)
 {
 
   // Initialize ImageMagick install location for Windows
-  InitializeMagick(*argv);
+  MagickPlusPlusGenesis genesis(*argv);
 
-  volatile int failures=0;
+  int failures=0;
 
   try {
 
