@@ -46,12 +46,17 @@ extern "C" {
 #define MagickPHI    1.61803398874989484820458683436563811772030917980576
 #define MagickPI2    1.57079632679489661923132169163975144209858469968755
 #define MagickPI     3.1415926535897932384626433832795028841971693993751058209749445923078164062
+#define MAGICK_PTRDIFF_MAX  (PTRDIFF_MAX)
+#define MAGICK_PTRDIFF_MIN  (-PTRDIFF_MAX-1)
 #define MagickSQ1_2  0.70710678118654752440084436210484903928483593768847
 #define MagickSQ2    1.41421356237309504880168872420969807856967187537695
 #define MagickSQ2PI  2.50662827463100024161235523934010416269302368164062
 #define MAGICK_SIZE_MAX  (SIZE_MAX)
 #define MAGICK_SSIZE_MAX  (SSIZE_MAX)
 #define MAGICK_SSIZE_MIN  (-SSIZE_MAX-1)
+#define MAGICK_UINT_MAX  (UINT_MAX)
+#define MAGICK_ULONG_MAX  (ULONG_MAX)
+#define MAGICK_USHORT_MAX  (USHRT_MAX)
 #define MatteColor  "#bdbdbd"  /* gray */
 #define MatteColorRGBA  ScaleShortToQuantum(0xbdbd),\
   ScaleShortToQuantum(0xbdbd),ScaleShortToQuantum(0xbdbd),OpaqueAlpha
@@ -60,11 +65,11 @@ extern "C" {
 #define SaveImagesTag  "Save/Images"
 #define SaveImageTag  "Save/Image"
 #define TransparentColor  "#00000000"  /* transparent black */
-#define TransparentColorRGBA  0,0,0,TransparentAlpha
+#define TransparentColorRGBA  (Quantum) 0,(Quantum) 0,(Quantum) 0,TransparentAlpha
 #define UndefinedCompressionQuality  0UL
 #define UndefinedTicksPerSecond  100L
 
-static inline ssize_t CastDoubleToLong(const double x)
+static inline ptrdiff_t CastDoubleToPtrdiffT(const double x)
 {
   double
     value;
@@ -74,48 +79,21 @@ static inline ssize_t CastDoubleToLong(const double x)
       errno=ERANGE;
       return(0);
     }
-  if (x < 0.0)
+  value=(x < 0.0) ? ceil(x) : floor(x);
+  if (value < ((double) MAGICK_PTRDIFF_MIN))
     {
-      value=ceil(x);
-      if (value < ((double) MAGICK_SSIZE_MIN))
-        {
-          errno=ERANGE;
-          return((ssize_t) MAGICK_SSIZE_MIN);
-        }
+      errno=ERANGE;
+      return(MAGICK_PTRDIFF_MIN);
     }
-  else
+  if (value > ((double) MAGICK_PTRDIFF_MAX))
     {
-      value=floor(x);
-      if (value > ((double) MAGICK_SSIZE_MAX))
-        {
-          errno=ERANGE;
-          return((ssize_t) MAGICK_SSIZE_MAX);
-        }
+      errno=ERANGE;
+      return(MAGICK_PTRDIFF_MAX);
     }
-  return((ssize_t) value);
+  return((ptrdiff_t) value);
 }
 
 static inline QuantumAny CastDoubleToQuantumAny(const double x)
-{
-  if (IsNaN(x) != 0)
-    {
-      errno=ERANGE;
-      return(0);
-    }
-  if (x > ((double) ((QuantumAny) ~0)))
-    {
-      errno=ERANGE;
-      return((QuantumAny) ~0);
-    }
-  if (x < 0.0)
-    {
-      errno=ERANGE;
-      return((QuantumAny) 0);
-    }
-  return((QuantumAny) (x+0.5));
-}
-
-static inline size_t CastDoubleToUnsigned(const double x)
 {
   double
     value;
@@ -125,20 +103,116 @@ static inline size_t CastDoubleToUnsigned(const double x)
       errno=ERANGE;
       return(0);
     }
-  value=floor(x);
-  if (value > ((double) MAGICK_SIZE_MAX))
-    {
-      errno=ERANGE;
-      return((size_t) MAGICK_SIZE_MAX);
-    }
+  value=(x < 0.0) ? ceil(x) : floor(x);
   if (value < 0.0)
     {
       errno=ERANGE;
       return(0);
     }
+  if (value > ((double) ((QuantumAny) ~0)))
+    {
+      errno=ERANGE;
+      return((QuantumAny) ~0);
+    }
+  return((QuantumAny) value);
+}
+
+static inline size_t CastDoubleToSizeT(const double x)
+{
+  double
+    value;
+
+  if (IsNaN(x) != 0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  value=(x < 0.0) ? ceil(x) : floor(x);
+  if (value < 0.0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  if (value > ((double) MAGICK_SIZE_MAX))
+    {
+      errno=ERANGE;
+      return(MAGICK_SIZE_MAX);
+    }
   return((size_t) value);
 }
 
+static inline ssize_t CastDoubleToSsizeT(const double x)
+{
+  double
+    value;
+
+  if (IsNaN(x) != 0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  value=(x < 0.0) ? ceil(x) : floor(x);
+  if (value < ((double) MAGICK_SSIZE_MIN))
+    {
+      errno=ERANGE;
+      return(MAGICK_SSIZE_MIN);
+    }
+  if (value > ((double) MAGICK_SSIZE_MAX))
+    {
+      errno=ERANGE;
+      return(MAGICK_SSIZE_MAX);
+    }
+  return((ssize_t) value);
+}
+
+static inline unsigned int CastDoubleToUInt(const double x)
+{
+  double
+    value;
+
+  if (IsNaN(x) != 0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  value=(x < 0.0) ? ceil(x) : floor(x);
+  if (value < 0.0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  if (value > ((double) MAGICK_UINT_MAX))
+    {
+      errno=ERANGE;
+      return(MAGICK_UINT_MAX);
+    }
+  return((unsigned int) value);
+}
+
+static inline unsigned short CastDoubleToUShort(const double x)
+{
+  double
+    value;
+
+  if (IsNaN(x) != 0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  value=(x < 0.0) ? ceil(x) : floor(x);
+  if (value < 0.0)
+    {
+      errno=ERANGE;
+      return(0);
+    }
+  if (value > ((double) MAGICK_USHORT_MAX))
+    {
+      errno=ERANGE;
+      return(MAGICK_USHORT_MAX);
+    }
+  return((unsigned short) value);
+}
+
 static inline double DegreesToRadians(const double degrees)
 {
   return((double) (MagickPI*degrees/180.0));
