@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -22,6 +22,7 @@
 #include "MagickCore/cache.h"
 #include "MagickCore/image-private.h"
 #include "MagickCore/pixel-accessor.h"
+#include "MagickCore/statistic-private.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -86,6 +87,9 @@ struct _QuantumInfo
 
   size_t
     signature;
+
+  size_t
+    meta_channel;
 };
 
 extern MagickPrivate void
@@ -110,11 +114,11 @@ static inline MagickSizeType GetQuantumRange(const size_t depth)
 static inline float HalfToSinglePrecision(const unsigned short half)
 {
 #define ExponentBias  (127-15)
-#define ExponentMask  0x7c00
+#define ExponentMask  (0x7c00U)
 #define ExponentShift  23
 #define SignBitShift  31
 #define SignificandShift  13
-#define SignificandMask  0x00000400
+#define SignificandMask  (0x00000400U)
 
   typedef union _SinglePrecision
   {
@@ -125,15 +129,13 @@ static inline float HalfToSinglePrecision(const unsigned short half)
       single_precision;
   } SinglePrecision;
 
-  register unsigned int
-    exponent,
-    significand,
-    sign_bit;
-
   SinglePrecision
     map;
 
   unsigned int
+    exponent,
+    significand,
+    sign_bit,
     value;
 
   /*
@@ -183,16 +185,16 @@ static inline float HalfToSinglePrecision(const unsigned short half)
 }
 
 static inline unsigned char *PopCharPixel(const unsigned char pixel,
-  unsigned char *pixels)
+  unsigned char *magick_restrict pixels)
 {
   *pixels++=pixel;
   return(pixels);
 }
 
 static inline unsigned char *PopLongPixel(const EndianType endian,
-  const unsigned int pixel,unsigned char *pixels)
+  const unsigned int pixel,unsigned char *magick_restrict pixels)
 {
-  register unsigned int
+  unsigned int
     quantum;
 
   quantum=(unsigned int) pixel;
@@ -212,9 +214,9 @@ static inline unsigned char *PopLongPixel(const EndianType endian,
 }
 
 static inline unsigned char *PopShortPixel(const EndianType endian,
-  const unsigned short pixel,unsigned char *pixels)
+  const unsigned short pixel,unsigned char *magick_restrict pixels)
 {
-  register unsigned int
+  unsigned int
     quantum;
 
   quantum=pixel;
@@ -229,17 +231,19 @@ static inline unsigned char *PopShortPixel(const EndianType endian,
   return(pixels);
 }
 
-static inline const unsigned char *PushCharPixel(const unsigned char *pixels,
-  unsigned char *pixel)
+static inline const unsigned char *PushCharPixel(
+  const unsigned char *magick_restrict pixels,
+  unsigned char *magick_restrict pixel)
 {
   *pixel=(*pixels++);
   return(pixels);
 }
 
 static inline const unsigned char *PushLongPixel(const EndianType endian,
-  const unsigned char *pixels,unsigned int *pixel)
+  const unsigned char *magick_restrict pixels,
+  unsigned int *magick_restrict pixel)
 {
-  register unsigned int
+  unsigned int
     quantum;
 
   if (endian == LSBEndian)
@@ -260,9 +264,10 @@ static inline const unsigned char *PushLongPixel(const EndianType endian,
 }
 
 static inline const unsigned char *PushShortPixel(const EndianType endian,
-  const unsigned char *pixels,unsigned short *pixel)
+  const unsigned char *magick_restrict pixels,
+  unsigned short *magick_restrict pixel)
 {
-  register unsigned int
+  unsigned int
     quantum;
 
   if (endian == LSBEndian)
@@ -279,7 +284,8 @@ static inline const unsigned char *PushShortPixel(const EndianType endian,
 }
 
 static inline const unsigned char *PushFloatPixel(const EndianType endian,
-  const unsigned char *pixels,MagickFloatType *pixel)
+  const unsigned char *magick_restrict pixels,
+  MagickFloatType *magick_restrict pixel)
 {
   union
   {
@@ -313,11 +319,11 @@ static inline Quantum ScaleAnyToQuantum(const QuantumAny quantum,
   if (quantum > range)
     return(QuantumRange);
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
-  return((Quantum) (((double) QuantumRange*quantum)*
-    PerceptibleReciprocal((double) range)+0.5));
+  return((Quantum) ((double) QuantumRange*(quantum*
+    MagickSafeReciprocal((double) range))+0.5));
 #else
-  return((Quantum) (((double) QuantumRange*quantum)*
-    PerceptibleReciprocal((double) range)));
+  return((Quantum) ((double) QuantumRange*(quantum*
+    MagickSafeReciprocal((double) range))));
 #endif
 }
 
@@ -327,11 +333,11 @@ static inline QuantumAny ScaleQuantumToAny(const Quantum quantum,
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((QuantumAny) ((double) range*quantum/QuantumRange));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0f))
     return((QuantumAny) 0UL);
-  if (((double) range*quantum/QuantumRange) >= 18446744073709551615.0)
+  if (((double) range*quantum/(double) QuantumRange) >= 18446744073709551615.0)
     return((QuantumAny) MagickULLConstant(18446744073709551615));
-  return((QuantumAny) ((double) range*quantum/QuantumRange+0.5));
+  return((QuantumAny) (range*(double) quantum/(double) QuantumRange+0.5));
 #endif
 }
 
@@ -377,7 +383,7 @@ static inline unsigned int ScaleQuantumToLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned int) (16843009UL*quantum));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0U);
   if ((16843009.0*quantum) >= 4294967295.0)
     return(4294967295UL);
@@ -390,11 +396,11 @@ static inline MagickSizeType ScaleQuantumToLongLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((MagickSizeType) (MagickULLConstant(72340172838076673)*quantum));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0UL);
   if ((72340172838076673.0*quantum) >= 18446744073709551615.0)
     return(MagickULLConstant(18446744073709551615));
-  return((MagickSizeType) (72340172838076673*quantum+0.5));
+  return((MagickSizeType) (72340172838076673.0*quantum+0.5));
 #endif
 }
 
@@ -405,7 +411,7 @@ static inline unsigned int ScaleQuantumToMap(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned int) quantum);
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0U);
   return((unsigned int) (quantum+0.5));
 #endif
@@ -416,7 +422,7 @@ static inline unsigned short ScaleQuantumToShort(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned short) (257UL*quantum));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0);
   if ((257.0*quantum) >= 65535.0)
     return(65535);
@@ -478,11 +484,11 @@ static inline unsigned int ScaleQuantumToLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned int) (65537UL*quantum));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0f))
     return(0U);
-  if ((65537.0*quantum) >= 4294967295.0)
+  if ((65537.0*(double) quantum) >= 4294967295.0)
     return(4294967295U);
-  return((unsigned int) (65537.0*quantum+0.5));
+  return((unsigned int) (65537.0*(double) quantum+0.5));
 #endif
 }
 
@@ -491,11 +497,11 @@ static inline MagickSizeType ScaleQuantumToLongLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((MagickSizeType) (MagickULLConstant(281479271743489)*quantum));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0f))
     return(0UL);
-  if ((281479271743489.0*quantum) >= 18446744073709551615.0)
+  if ((281479271743489.0*(double) quantum) >= 18446744073709551615.0)
     return(MagickULLConstant(18446744073709551615));
-  return((MagickSizeType) (281479271743489.0*quantum+0.5));
+  return((MagickSizeType) (281479271743489.0*(double) quantum+0.5));
 #endif
 }
 
@@ -506,9 +512,9 @@ static inline unsigned int ScaleQuantumToMap(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned int) quantum);
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0f))
     return(0U);
-  return((unsigned int) (quantum+0.5));
+  return((unsigned int) (quantum+0.5f));
 #endif
 }
 
@@ -517,11 +523,11 @@ static inline unsigned short ScaleQuantumToShort(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned short) quantum);
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0f))
     return(0);
-  if (quantum >= 65535.0)
+  if (quantum >= 65535.0f)
     return(65535);
-  return((unsigned short) (quantum+0.5));
+  return((unsigned short) (quantum+0.5f));
 #endif
 }
 
@@ -571,7 +577,7 @@ static inline unsigned int ScaleQuantumToLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((unsigned int) quantum);
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0U);
   if ((quantum) >= 4294967295.0)
     return(4294967295);
@@ -584,7 +590,7 @@ static inline MagickSizeType ScaleQuantumToLongLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((MagickSizeType) (MagickULLConstant(4294967297)*quantum));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0UL);
   if ((4294967297.0*quantum) >= 18446744073709551615.0)
     return(MagickULLConstant(18446744073709551615));
@@ -600,7 +606,7 @@ static inline unsigned int ScaleQuantumToMap(const Quantum quantum)
   return((unsigned int) ((quantum+MagickULLConstant(32768))/
     MagickULLConstant(65537)));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0U);
   return((unsigned int) (quantum/65537.0+0.5));
 #endif
@@ -612,7 +618,7 @@ static inline unsigned short ScaleQuantumToShort(const Quantum quantum)
   return((unsigned short) ((quantum+MagickULLConstant(32768))/
     MagickULLConstant(65537)));
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0);
   if ((quantum/65537.0) >= 65535.0)
     return(65535);
@@ -663,7 +669,7 @@ static inline MagickSizeType ScaleQuantumToLongLong(const Quantum quantum)
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   return((MagickSizeType) quantum);
 #else
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0UL);
   if (quantum >= 18446744073709551615.0)
     return(MagickULLConstant(18446744073709551615));
@@ -673,7 +679,7 @@ static inline MagickSizeType ScaleQuantumToLongLong(const Quantum quantum)
 
 static inline unsigned int ScaleQuantumToMap(const Quantum quantum)
 {
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0U);
   if ((quantum/281479271743489.0) >= MaxMap)
     return((unsigned int) MaxMap);
@@ -682,7 +688,7 @@ static inline unsigned int ScaleQuantumToMap(const Quantum quantum)
 
 static inline unsigned short ScaleQuantumToShort(const Quantum quantum)
 {
-  if ((IsNaN(quantum) != MagickFalse) || (quantum <= 0.0))
+  if ((IsNaN(quantum) != 0) || (quantum <= 0.0))
     return(0);
   if ((quantum/281479271743489.0) >= 65535.0)
     return(65535);
@@ -695,7 +701,7 @@ static inline Quantum ScaleShortToQuantum(const unsigned short value)
 }
 #endif
 
-static inline unsigned short SinglePrecisionToHalf(const float value)
+static inline unsigned short SinglePrecisionToHalf(const double value)
 {
   typedef union _SinglePrecision
   {
@@ -706,16 +712,16 @@ static inline unsigned short SinglePrecisionToHalf(const float value)
       single_precision;
   } SinglePrecision;
 
-  register int
+  int
     exponent;
 
-  register unsigned int
-    significand,
-    sign_bit;
-
   SinglePrecision
     map;
 
+  unsigned int
+    significand,
+    sign_bit;
+
   unsigned short
     half;
 
@@ -726,7 +732,7 @@ static inline unsigned short SinglePrecisionToHalf(const float value)
       Exponent width: 5 bits
       Significand precision: 11 (10 explicitly stored)
   */
-  map.single_precision=value;
+  map.single_precision=(float) value;
   sign_bit=(map.fixed_point >> 16) & 0x00008000;
   exponent=(int) ((map.fixed_point >> ExponentShift) & 0x000000ff)-ExponentBias;
   significand=map.fixed_point & 0x007fffff;
@@ -739,7 +745,7 @@ static inline unsigned short SinglePrecisionToHalf(const float value)
         return((unsigned short) sign_bit);
       significand=significand | 0x00800000;
       shift=(int) (14-exponent);
-      significand=(unsigned int) ((significand+((1 << (shift-1))-1)+
+      significand=(unsigned int) ((significand+((1U << (shift-1))-1)+
         ((significand >> shift) & 0x01)) >> shift);
       return((unsigned short) (sign_bit | significand));
     }
@@ -767,7 +773,7 @@ static inline unsigned short SinglePrecisionToHalf(const float value)
       float
         alpha;
 
-      register int
+      int
         i;
 
       /*
@@ -778,7 +784,7 @@ static inline unsigned short SinglePrecisionToHalf(const float value)
         alpha*=alpha;
       return((unsigned short) (sign_bit | ExponentMask));
     }
-  half=(unsigned short) (sign_bit | (exponent << 10) |
+  half=(unsigned short) (sign_bit | ((unsigned int) exponent << 10) |
     (significand >> SignificandShift));
   return(half);
 }
