@@ -17,7 +17,7 @@
 %                                 May 2008                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2008 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -62,7 +62,7 @@
 #include "MagickCore/module.h"
 
 /* 
-Tyedef declarations
+Typedef declarations
  */
 
 typedef struct _IPLInfo
@@ -203,11 +203,11 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if ( image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent, GetMagickModule(), "%s",
-                image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -300,7 +300,12 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       quantum_format = UnsignedQuantumFormat;
       break;
   }
-  extent=ipl_info.width*ipl_info.height*ipl_info.z*ipl_info.depth/8;
+  if (HeapOverflowSanityCheckGetSize(ipl_info.width,ipl_info.height,&extent) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (HeapOverflowSanityCheckGetSize(extent,ipl_info.z,&extent) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (HeapOverflowSanityCheckGetSize(extent,ipl_info.depth/8,&extent) != MagickFalse)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if (extent > GetBlobSize(image))
     ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
 
@@ -327,23 +332,13 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status=SetImageExtent(image,image->columns,image->rows,exception);
     if (status == MagickFalse)
       return(DestroyImageList(image));
-/*
-   printf("Length: %.20g, Memory size: %.20g\n", (double) length,(double)
-     image->depth);
-*/
      quantum_info=AcquireQuantumInfo(image_info,image);
      if (quantum_info == (QuantumInfo *) NULL)
        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
      status=SetQuantumFormat(image,quantum_info,quantum_format);
      if (status == MagickFalse)
        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-     pixels=(unsigned char *) GetQuantumPixels(quantum_info); 
-     if(image->columns != ipl_info.width){
-/*
-     printf("Columns not set correctly!  Wanted: %.20g, got: %.20g\n",
-       (double) ipl_info.width, (double) image->columns);
-*/
-     }
+     pixels=(unsigned char *) GetQuantumPixels(quantum_info);
 
     /* 
     Covert IPL binary to pixel packets
@@ -529,7 +524,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
     *quantum_info;
 
   size_t
-    imageListLength;
+    number_scenes;
 
   ssize_t
     y;
@@ -544,10 +539,10 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
@@ -587,8 +582,8 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
     break;
     
   }
-  imageListLength=GetImageListLength(image);
-  ipl_info.z = (unsigned int) imageListLength;
+  number_scenes=GetImageListLength(image);
+  ipl_info.z = (unsigned int) number_scenes;
   /* There is no current method for detecting whether we have T or Z stacks */
   ipl_info.time = 1;
   ipl_info.width = (unsigned int) image->columns;
@@ -688,7 +683,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
   if (GetNextImageInList(image) == (Image *) NULL)
     break;
       image=SyncNextImageInList(image);
-      status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
+      status=SetImageProgress(image,SaveImagesTag,scene++,number_scenes);
       if (status == MagickFalse)
         break;
     }while (image_info->adjoin != MagickFalse);
