@@ -122,19 +122,12 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     status;
 
   Quantum
-    index;
-
-  ssize_t
-    x;
-
-  Quantum
+    index,
     *q;
 
-  unsigned char
-    *p;
-
   ssize_t
     length,
+    x,
     y;
 
   struct jbg_dec_state
@@ -143,7 +136,8 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   unsigned char
     bit,
     *buffer,
-    byte;
+    byte,
+    *p;
 
   /*
     Open image file.
@@ -165,6 +159,9 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   /*
     Initialize JBIG toolkit.
   */
+  if ((image->columns != (unsigned long) image->columns) ||
+      (image->rows != (unsigned long) image->rows))
+    ThrowReaderException(ImageError,"WidthOrHeightExceedsLimit");
   jbg_dec_init(&jbig_info);
   jbg_dec_maxsize(&jbig_info,(unsigned long) image->columns,(unsigned long)
     image->rows);
@@ -196,7 +193,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
         count;
 
       status=(MagickStatusType) jbg_dec_in(&jbig_info,p,(size_t) length,&count);
-      p+=count;
+      p+=(ptrdiff_t) count;
       length-=(ssize_t) count;
     }
   } while ((status == JBG_EAGAIN) || (status == JBG_EOK));
@@ -249,14 +246,14 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
     {
       if (bit == 0)
         byte=(*p++);
-      index=(byte & 0x80) ? 0 : 1;
+      index=(Quantum) ((byte & 0x80) ? 0 : 1);
       bit++;
       byte<<=1;
       if (bit == 8)
         bit=0;
       SetPixelIndex(image,index,q);
       SetPixelViaPixelInfo(image,image->colormap+(ssize_t) index,q);
-      q+=GetPixelChannels(image);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
@@ -410,6 +407,9 @@ static void JBIGEncode(unsigned char *pixels,size_t length,void *data)
 static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   Image *image,ExceptionInfo *exception)
 {
+  const Quantum
+    *p;
+
   double
     version;
 
@@ -422,20 +422,12 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   MemoryInfo
     *pixel_info;
 
-  const Quantum
-    *p;
-
-  ssize_t
-    x;
-
-  unsigned char
-    *q;
-
   size_t
     number_images,
     number_packets;
 
   ssize_t
+    x,
     y;
 
   struct jbg_enc_state
@@ -444,7 +436,8 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   unsigned char
     bit,
     byte,
-    *pixels;
+    *pixels,
+    *q;
 
   /*
     Open image file.
@@ -499,7 +492,7 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
             bit=0;
             byte=0;
           }
-        p+=GetPixelChannels(image);
+        p+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (bit != 0)
         *q++=byte << (8-bit);
@@ -536,15 +529,15 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
               flags;
 
             flags=ParseGeometry(image_info->density,&geometry_info);
-            x_resolution=geometry_info.rho;
-            y_resolution=geometry_info.sigma;
+            x_resolution=(size_t) geometry_info.rho;
+            y_resolution=(size_t) geometry_info.sigma;
             if ((flags & SigmaValue) == 0)
               y_resolution=x_resolution;
           }
         if (image->units == PixelsPerCentimeterResolution)
           {
-            x_resolution=(size_t) (100.0*2.54*x_resolution+0.5)/100.0;
-            y_resolution=(size_t) (100.0*2.54*y_resolution+0.5)/100.0;
+            x_resolution=(size_t) ((100.0*2.54*x_resolution+0.5)/100.0);
+            y_resolution=(size_t) ((100.0*2.54*y_resolution+0.5)/100.0);
           }
         (void) jbg_enc_lrlmax(&jbig_info,(unsigned long) x_resolution,
           (unsigned long) y_resolution);
