@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -61,7 +61,7 @@
 #include "MagickCore/monitor.h"
 #include "MagickCore/monitor-private.h"
 #include "MagickCore/pixel-accessor.h"
-#include "MagickCore/profile.h"
+#include "MagickCore/profile-private.h"
 #include "MagickCore/resource_.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
@@ -121,7 +121,7 @@ static const PICTCode
   codes[] =
   {
     /* 0x00 */ { "NOP", 0, "nop" },
-    /* 0x01 */ { "Clip", 0, "clip" },
+    /* 0x01 */ { "ClipRgn", 0, "clip" },
     /* 0x02 */ { "BkPat", 8, "background pattern" },
     /* 0x03 */ { "TxFont", 2, "text font (word)" },
     /* 0x04 */ { "TxFace", 1, "text face (byte)" },
@@ -474,7 +474,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       */
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        q=pixels+y*width*GetPixelChannels(image);
+        q=pixels+y*(ssize_t) width*(ssize_t) GetPixelChannels(image);
         number_pixels=bytes_per_line;
         count=ReadBlob(blob,(size_t) number_pixels,scanline);
         if (count != (ssize_t) number_pixels)
@@ -500,8 +500,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   */
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    q=pixels+y*width;
-    if (bytes_per_line > 200)
+    q=pixels+y*(ssize_t) width;
+    if (bytes_per_line > 250)
       scanline_length=ReadBlobMSBShort(blob);
     else
       scanline_length=(size_t) ReadBlobByte(blob);
@@ -523,9 +523,9 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
           number_pixels=length*bytes_per_pixel;
           p=UnpackScanline(scanline+j+1,bits_per_pixel,unpack_buffer,
             &number_pixels);
-          if ((q-pixels+number_pixels) <= *extent)
+          if ((size_t) (q-pixels+(ssize_t) number_pixels) <= *extent)
             (void) memcpy(q,p,(size_t) number_pixels);
-          q+=number_pixels;
+          q+=(ptrdiff_t) number_pixels;
           j+=(ssize_t) (length*bytes_per_pixel+1);
         }
       else
@@ -536,9 +536,9 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
             &number_pixels);
           for (i=0; i < (ssize_t) length; i++)
           {
-            if ((q-pixels+number_pixels) <= *extent)
+            if ((size_t) (q-pixels+(ssize_t) number_pixels) <= *extent)
               (void) memcpy(q,p,(size_t) number_pixels);
-            q+=number_pixels;
+            q+=(ptrdiff_t) number_pixels;
           }
           j+=(ssize_t) bytes_per_pixel+1;
         }
@@ -611,10 +611,10 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(scanline != (unsigned char *) NULL);
   assert(pixels != (unsigned char *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   count=0;
   runlength=0;
   p=scanline+(bytes_per_line-1);
@@ -691,7 +691,7 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
     Write the number of and the packed length.
   */
   length=(size_t) (q-pixels);
-  if (bytes_per_line > 200)
+  if (bytes_per_line > 250)
     {
       (void) WriteBlobMSBShort(image,(unsigned short) length);
       length+=2;
@@ -829,7 +829,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
 }
 
   char
-    geometry[MagickPathExtent],
     header_ole[4];
 
   Image
@@ -847,6 +846,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     jpeg,
     status;
 
+  MagickSizeType
+    blob_size,
+    size;
+
   PICTRectangle
     frame;
 
@@ -882,7 +885,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
@@ -904,16 +907,25 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   /*
     Skip header : 512 for standard PICT and 4, ie "PICT" for OLE2.
   */
-  header_ole[0]=ReadBlobByte(image);
-  header_ole[1]=ReadBlobByte(image);
-  header_ole[2]=ReadBlobByte(image);
-  header_ole[3]=ReadBlobByte(image);
-  if (!((header_ole[0] == 0x50) && (header_ole[1] == 0x49) &&
-      (header_ole[2] == 0x43) && (header_ole[3] == 0x54 )))
-    for (i=0; i < 508; i++)
-      if (ReadBlobByte(image) == EOF)
-        break;
-  (void) ReadBlobMSBShort(image);  /* skip picture size */
+  header_ole[0]=(char) ReadBlobByte(image);
+  header_ole[1]=(char) ReadBlobByte(image);
+  size=(((MagickSizeType) header_ole[0]) << 8) | (MagickSizeType) header_ole[1];
+  blob_size=GetBlobSize(image);
+  /*
+    This is a special case where the PICT file directly starts with the picture size.
+  */
+  if ((blob_size % ((MagickSizeType) 65536)) != size)
+    {
+      char
+        buffer[508];
+
+      header_ole[2]=(char) ReadBlobByte(image);
+      header_ole[3]=(char) ReadBlobByte(image);
+      if (!((header_ole[0] == 0x50) && (header_ole[1] == 0x49) &&
+          (header_ole[2] == 0x43) && (header_ole[3] == 0x54 )))
+        ReadBlob(image,sizeof(buffer),buffer);
+      (void) ReadBlobMSBShort(image);  /* skip picture size */
+    }
   if (ReadRectangle(image,&frame) == MagickFalse)
     ThrowPICTException(CorruptImageError,"ImproperImageHeader");
   while ((c=ReadBlobByte(image)) == 0) ;
@@ -1084,10 +1096,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
 
                 if (EOFBlob(image) != MagickFalse)
                   break;
-                if (length > 200)
+                if (length > 250)
                   scanline_length=ReadBlobMSBShort(image);
                 else
-                  scanline_length=ReadBlobByte(image);
+                  scanline_length=(size_t) ReadBlobByte(image);
                 if ((MagickSizeType) scanline_length > GetBlobSize(image))
                   ThrowPICTException(CorruptImageError,
                     "InsufficientImageDataInFile");
@@ -1226,11 +1238,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   {
                     for (i=0; i < (ssize_t) tile_image->colors; i++)
                     {
-                      tile_image->colormap[i].red=(Quantum) (QuantumRange-
+                      tile_image->colormap[i].red=((double) QuantumRange-
                         tile_image->colormap[i].red);
-                      tile_image->colormap[i].green=(Quantum) (QuantumRange-
+                      tile_image->colormap[i].green=((double) QuantumRange-
                         tile_image->colormap[i].green);
-                      tile_image->colormap[i].blue=(Quantum) (QuantumRange-
+                      tile_image->colormap[i].blue=((double) QuantumRange-
                         tile_image->colormap[i].blue);
                     }
                   }
@@ -1288,11 +1300,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                       *p,exception);
                     SetPixelIndex(tile_image,index,q);
                     SetPixelRed(tile_image,
-                      tile_image->colormap[(ssize_t) index].red,q);
+                      (Quantum) tile_image->colormap[(ssize_t) index].red,q);
                     SetPixelGreen(tile_image,
-                      tile_image->colormap[(ssize_t) index].green,q);
+                      (Quantum) tile_image->colormap[(ssize_t) index].green,q);
                     SetPixelBlue(tile_image,
-                      tile_image->colormap[(ssize_t) index].blue,q);
+                      (Quantum) tile_image->colormap[(ssize_t) index].blue,q);
                   }
                 else
                   {
@@ -1303,7 +1315,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                         SetPixelRed(tile_image,ScaleCharToQuantum(
                           (unsigned char) ((i & 0x7c) << 1)),q);
                         SetPixelGreen(tile_image,ScaleCharToQuantum(
-                          (unsigned char) (((i & 0x03) << 6) |
+                          (unsigned char) ((size_t) ((i & 0x03) << 6) |
                           ((k & 0xe0) >> 2))),q);
                         SetPixelBlue(tile_image,ScaleCharToQuantum(
                           (unsigned char) ((k & 0x1f) << 3)),q);
@@ -1335,14 +1347,15 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                         }
                   }
                 p++;
-                q+=GetPixelChannels(tile_image);
+                q+=(ptrdiff_t) GetPixelChannels(tile_image);
               }
               if (SyncAuthenticPixels(tile_image,exception) == MagickFalse)
                 break;
               if ((tile_image->storage_class == DirectClass) &&
                   (pixmap.bits_per_pixel != 16))
                 {
-                  p+=(pixmap.component_count-1)*tile_image->columns;
+                  p+=(ptrdiff_t) (pixmap.component_count-1)*(ssize_t)
+                    tile_image->columns;
                   if (p < pixels)
                     break;
                 }
@@ -1355,9 +1368,24 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if ((jpeg == MagickFalse) && (EOFBlob(image) == MagickFalse))
               if ((code == 0x9a) || (code == 0x9b) ||
                   ((bytes_per_line & 0x8000) != 0))
+                {
+                  if ((source.right-source.left) != (destination.right-destination.left) ||
+                      (source.bottom-source.top) != (destination.bottom-destination.top))
+                    {
+                      Image *clone_image = tile_image;
+                      tile_image=ResizeImage(clone_image,(size_t)
+                        (destination.right-destination.left),(size_t)
+                        (destination.bottom-destination.top),UndefinedFilter,
+                        exception);
+                      if (tile_image == (Image *) NULL)
+                        tile_image=clone_image;
+                      else
+                        clone_image=DestroyImage(clone_image);
+                    }
                 (void) CompositeImage(image,tile_image,CopyCompositeOp,
                   MagickTrue,(ssize_t) destination.left,(ssize_t)
                   destination.top,exception);
+              }
             tile_image=DestroyImage(tile_image);
             break;
           }
@@ -1396,30 +1424,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             {
               case 0xe0:
               {
-                profile=BlobToStringInfo((const void *) NULL,length);
-                SetStringInfoDatum(profile,info);
-                status=SetImageProfile(image,"icc",profile,exception);
-                profile=DestroyStringInfo(profile);
-                if (status == MagickFalse)
-                  {
-                    info=(unsigned char *) RelinquishMagickMemory(info);
-                    ThrowPICTException(ResourceLimitError,
-                      "MemoryAllocationFailed");
-                  }
+                profile=BlobToProfileStringInfo("icc",info,length,exception);
+                (void) SetImageProfilePrivate(image,profile,exception);
                 break;
               }
               case 0x1f2:
               {
-                profile=BlobToStringInfo((const void *) NULL,length);
-                SetStringInfoDatum(profile,info);
-                status=SetImageProfile(image,"iptc",profile,exception);
-                if (status == MagickFalse)
-                  {
-                    info=(unsigned char *) RelinquishMagickMemory(info);
-                    ThrowPICTException(ResourceLimitError,
-                      "MemoryAllocationFailed");
-                  }
-                profile=DestroyStringInfo(profile);
+                profile=BlobToProfileStringInfo("iptc",info,length,exception);
+                (void) SetImageProfilePrivate(image,profile,exception);
                 break;
               }
               default:
@@ -1493,15 +1505,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             read_info=AcquireImageInfo();
             (void) FormatLocaleString(read_info->filename,MagickPathExtent,
               "jpeg:%s",image_info->filename);
-            tile_image=BlobToImage(read_info,stream,count,exception);
+            tile_image=BlobToImage(read_info,stream,(size_t) count,exception);
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             read_info=DestroyImageInfo(read_info);
           }
         if (tile_image == (Image *) NULL)
           continue;
-        (void) FormatLocaleString(geometry,MagickPathExtent,"%.20gx%.20g",
-          (double) MagickMax(image->columns,tile_image->columns),
-          (double) MagickMax(image->rows,tile_image->rows));
         (void) SetImageExtent(image,
           MagickMax(image->columns,tile_image->columns),
           MagickMax(image->rows,tile_image->rows),exception);
@@ -1544,7 +1553,10 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         continue;
       }
   }
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -1659,12 +1671,15 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
 #define PictPICTOp  0x98
 #define PictVersion  0x11
 
+  const Quantum
+    *p;
+
   const StringInfo
     *profile;
 
   double
-    x_resolution,
-    y_resolution;
+    x_resolution = DefaultResolution,
+    y_resolution = DefaultResolution;
 
   MagickBooleanType
     status;
@@ -1683,13 +1698,6 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     size_rectangle,
     source_rectangle;
 
-  const Quantum
-    *p;
-
-  ssize_t
-    i,
-    x;
-
   size_t
     bytes_per_line,
     count,
@@ -1697,6 +1705,8 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     storage_class;
 
   ssize_t
+    i,
+    x,
     y;
 
   unsigned char
@@ -1715,12 +1725,12 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->columns > 65535L) || (image->rows > 65535L))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickCoreSignature);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
@@ -1754,8 +1764,6 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   pixmap.table=0;
   pixmap.reserved=0;
   transfer_mode=0;
-  x_resolution=0.0;
-  y_resolution=0.0;
   if ((image->resolution.x > MagickEpsilon) &&
       (image->resolution.y > MagickEpsilon))
     {
@@ -1998,11 +2006,11 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       {
         (void) WriteBlobMSBShort(image,(unsigned short) i);
         (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
-          image->colormap[i].red));
+          (Quantum) image->colormap[i].red));
         (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
-          image->colormap[i].green));
+          (Quantum) image->colormap[i].green));
         (void) WriteBlobMSBShort(image,ScaleQuantumToShort(
-          image->colormap[i].blue));
+          (Quantum) image->colormap[i].blue));
       }
     }
   /*
@@ -2030,7 +2038,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       for (x=0; x < (ssize_t) image->columns; x++)
       {
         scanline[x]=(unsigned char) ((ssize_t) GetPixelIndex(image,p));
-        p+=GetPixelChannels(image);
+        p+=(ptrdiff_t) GetPixelChannels(image);
       }
       count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
         packed_scanline);
@@ -2084,7 +2092,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
             *blue++=ScaleQuantumToChar(GetPixelBlue(image,p));
             if (image->alpha_trait != UndefinedPixelTrait)
               *opacity++=ScaleQuantumToChar((Quantum) (GetPixelAlpha(image,p)));
-            p+=GetPixelChannels(image);
+            p+=(ptrdiff_t) GetPixelChannels(image);
           }
           count+=EncodeImage(image,scanline,bytes_per_line,packed_scanline);
           if (image->previous == (Image *) NULL)
@@ -2105,6 +2113,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   packed_scanline=(unsigned char *) RelinquishMagickMemory(packed_scanline);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
